# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x63ogment/api/common.proto\x12\ncogmentAPI\x1a\x19google/protobuf/any.proto\"\x10\n\x0eVersionRequest\"j\n\x0bVersionInfo\x12\x31\n\x08versions\x18\x01 \x03(\x0b\x32\x1f.cogmentAPI.VersionInfo.Version\x1a(\n\x07Version\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"$\n\x11SerializedMessage\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"9\n\rDatalogParams\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x16\n\x0e\x65xclude_fields\x18\x02 \x03(\t\"z\n\x11\x45nvironmentParams\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x02 \x01(\t\x12\x16\n\x0eimplementation\x18\x03 \x01(\t\x12-\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessage\"\x90\x02\n\x0b\x41\x63torParams\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x61\x63tor_class\x18\x02 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x03 \x01(\t\x12\x16\n\x0eimplementation\x18\x04 \x01(\t\x12-\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessage\x12\"\n\x1ainitial_connection_timeout\x18\x06 \x01(\x02\x12\x18\n\x10response_timeout\x18\x07 \x01(\x02\x12\x10\n\x08optional\x18\x08 \x01(\x08\x12\x35\n\x0e\x64\x65\x66\x61ult_action\x18\t \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessage\"\x91\x02\n\x0bTrialParams\x12\x33\n\x0ctrial_config\x18\x01 \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessage\x12*\n\x07\x64\x61talog\x18\x02 \x01(\x0b\x32\x19.cogmentAPI.DatalogParams\x12\x32\n\x0b\x65nvironment\x18\x03 \x01(\x0b\x32\x1d.cogmentAPI.EnvironmentParams\x12\'\n\x06\x61\x63tors\x18\x04 \x03(\x0b\x32\x17.cogmentAPI.ActorParams\x12\x11\n\tmax_steps\x18\x05 \x01(\r\x12\x16\n\x0emax_inactivity\x18\x06 \x01(\r\x12\x19\n\x11nb_buffered_ticks\x18\x07 \x01(\x12\"/\n\nTrialActor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x61\x63tor_class\x18\x02 \x01(\t\"B\n\x0bObservation\x12\x0f\n\x07tick_id\x18\x01 \x01(\x04\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\"=\n\x06\x41\x63tion\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\"o\n\x0cRewardSource\x12\x13\n\x0bsender_name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02\x12\x12\n\nconfidence\x18\x03 \x01(\x02\x12\'\n\tuser_data\x18\x04 \x01(\x0b\x32\x14.google.protobuf.Any\"j\n\x06Reward\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x15\n\rreceiver_name\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x02\x12)\n\x07sources\x18\x04 \x03(\x0b\x32\x18.cogmentAPI.RewardSource\"m\n\x07Message\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x13\n\x0bsender_name\x18\x02 \x01(\t\x12\x15\n\rreceiver_name\x18\x03 \x01(\t\x12%\n\x07payload\x18\x04 \x01(\x0b\x32\x14.google.protobuf.Any\"\x91\x02\n\x12\x41\x63torRunTrialInput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x33\n\ninit_input\x18\x02 \x01(\x0b\x32\x1d.cogmentAPI.ActorInitialInputH\x00\x12.\n\x0bobservation\x18\x03 \x01(\x0b\x32\x17.cogmentAPI.ObservationH\x00\x12$\n\x06reward\x18\x04 \x01(\x0b\x32\x12.cogmentAPI.RewardH\x00\x12&\n\x07message\x18\x05 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x06 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\x8a\x02\n\x13\x41\x63torRunTrialOutput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x35\n\x0binit_output\x18\x02 \x01(\x0b\x32\x1e.cogmentAPI.ActorInitialOutputH\x00\x12$\n\x06\x61\x63tion\x18\x03 \x01(\x0b\x32\x12.cogmentAPI.ActionH\x00\x12$\n\x06reward\x18\x04 \x01(\x0b\x32\x12.cogmentAPI.RewardH\x00\x12&\n\x07message\x18\x05 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x06 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\x90\x01\n\x11\x41\x63torInitialInput\x12\x12\n\nactor_name\x18\x01 \x01(\t\x12\x13\n\x0b\x61\x63tor_class\x18\x02 \x01(\t\x12\x11\n\timpl_name\x18\x03 \x01(\t\x12\x10\n\x08\x65nv_name\x18\x04 \x01(\t\x12-\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessage\"S\n\x12\x41\x63torInitialOutput\x12\x15\n\x0b\x61\x63tor_class\x18\x01 \x01(\tH\x00\x12\x14\n\nactor_name\x18\x02 \x01(\tH\x00\x42\x10\n\x0eslot_selection\"\x1e\n\x0bTrialConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"\x1e\n\x0b\x41\x63torConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"$\n\x11\x45nvironmentConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c*a\n\nTrialState\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0cINITIALIZING\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x12\x0b\n\x07RUNNING\x10\x03\x12\x0f\n\x0bTERMINATING\x10\x04\x12\t\n\x05\x45NDED\x10\x05*g\n\x12\x43ommunicationState\x12\x15\n\x11UNKNOWN_COM_STATE\x10\x00\x12\n\n\x06NORMAL\x10\x01\x12\r\n\tHEARTBEAT\x10\x02\x12\x08\n\x04LAST\x10\x03\x12\x0c\n\x08LAST_ACK\x10\x04\x12\x07\n\x03\x45ND\x10\x05\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')

_TRIALSTATE = DESCRIPTOR.enum_types_by_name['TrialState']
TrialState = enum_type_wrapper.EnumTypeWrapper(_TRIALSTATE)
_COMMUNICATIONSTATE = DESCRIPTOR.enum_types_by_name['CommunicationState']
CommunicationState = enum_type_wrapper.EnumTypeWrapper(_COMMUNICATIONSTATE)
UNKNOWN = 0
INITIALIZING = 1
PENDING = 2
RUNNING = 3
TERMINATING = 4
ENDED = 5
UNKNOWN_COM_STATE = 0
NORMAL = 1
HEARTBEAT = 2
LAST = 3
LAST_ACK = 4
END = 5


_VERSIONREQUEST = DESCRIPTOR.message_types_by_name['VersionRequest']
_VERSIONINFO = DESCRIPTOR.message_types_by_name['VersionInfo']
_VERSIONINFO_VERSION = _VERSIONINFO.nested_types_by_name['Version']
_SERIALIZEDMESSAGE = DESCRIPTOR.message_types_by_name['SerializedMessage']
_DATALOGPARAMS = DESCRIPTOR.message_types_by_name['DatalogParams']
_ENVIRONMENTPARAMS = DESCRIPTOR.message_types_by_name['EnvironmentParams']
_ACTORPARAMS = DESCRIPTOR.message_types_by_name['ActorParams']
_TRIALPARAMS = DESCRIPTOR.message_types_by_name['TrialParams']
_TRIALACTOR = DESCRIPTOR.message_types_by_name['TrialActor']
_OBSERVATION = DESCRIPTOR.message_types_by_name['Observation']
_ACTION = DESCRIPTOR.message_types_by_name['Action']
_REWARDSOURCE = DESCRIPTOR.message_types_by_name['RewardSource']
_REWARD = DESCRIPTOR.message_types_by_name['Reward']
_MESSAGE = DESCRIPTOR.message_types_by_name['Message']
_ACTORRUNTRIALINPUT = DESCRIPTOR.message_types_by_name['ActorRunTrialInput']
_ACTORRUNTRIALOUTPUT = DESCRIPTOR.message_types_by_name['ActorRunTrialOutput']
_ACTORINITIALINPUT = DESCRIPTOR.message_types_by_name['ActorInitialInput']
_ACTORINITIALOUTPUT = DESCRIPTOR.message_types_by_name['ActorInitialOutput']
_TRIALCONFIG = DESCRIPTOR.message_types_by_name['TrialConfig']
_ACTORCONFIG = DESCRIPTOR.message_types_by_name['ActorConfig']
_ENVIRONMENTCONFIG = DESCRIPTOR.message_types_by_name['EnvironmentConfig']
VersionRequest = _reflection.GeneratedProtocolMessageType('VersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERSIONREQUEST,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.VersionRequest)
  })
_sym_db.RegisterMessage(VersionRequest)

VersionInfo = _reflection.GeneratedProtocolMessageType('VersionInfo', (_message.Message,), {

  'Version' : _reflection.GeneratedProtocolMessageType('Version', (_message.Message,), {
    'DESCRIPTOR' : _VERSIONINFO_VERSION,
    '__module__' : 'cogment.api.common_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.VersionInfo.Version)
    })
  ,
  'DESCRIPTOR' : _VERSIONINFO,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.VersionInfo)
  })
_sym_db.RegisterMessage(VersionInfo)
_sym_db.RegisterMessage(VersionInfo.Version)

SerializedMessage = _reflection.GeneratedProtocolMessageType('SerializedMessage', (_message.Message,), {
  'DESCRIPTOR' : _SERIALIZEDMESSAGE,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.SerializedMessage)
  })
_sym_db.RegisterMessage(SerializedMessage)

DatalogParams = _reflection.GeneratedProtocolMessageType('DatalogParams', (_message.Message,), {
  'DESCRIPTOR' : _DATALOGPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DatalogParams)
  })
_sym_db.RegisterMessage(DatalogParams)

EnvironmentParams = _reflection.GeneratedProtocolMessageType('EnvironmentParams', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvironmentParams)
  })
_sym_db.RegisterMessage(EnvironmentParams)

ActorParams = _reflection.GeneratedProtocolMessageType('ActorParams', (_message.Message,), {
  'DESCRIPTOR' : _ACTORPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorParams)
  })
_sym_db.RegisterMessage(ActorParams)

TrialParams = _reflection.GeneratedProtocolMessageType('TrialParams', (_message.Message,), {
  'DESCRIPTOR' : _TRIALPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialParams)
  })
_sym_db.RegisterMessage(TrialParams)

TrialActor = _reflection.GeneratedProtocolMessageType('TrialActor', (_message.Message,), {
  'DESCRIPTOR' : _TRIALACTOR,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialActor)
  })
_sym_db.RegisterMessage(TrialActor)

Observation = _reflection.GeneratedProtocolMessageType('Observation', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Observation)
  })
_sym_db.RegisterMessage(Observation)

Action = _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
  'DESCRIPTOR' : _ACTION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Action)
  })
_sym_db.RegisterMessage(Action)

RewardSource = _reflection.GeneratedProtocolMessageType('RewardSource', (_message.Message,), {
  'DESCRIPTOR' : _REWARDSOURCE,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RewardSource)
  })
_sym_db.RegisterMessage(RewardSource)

Reward = _reflection.GeneratedProtocolMessageType('Reward', (_message.Message,), {
  'DESCRIPTOR' : _REWARD,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Reward)
  })
_sym_db.RegisterMessage(Reward)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Message)
  })
_sym_db.RegisterMessage(Message)

ActorRunTrialInput = _reflection.GeneratedProtocolMessageType('ActorRunTrialInput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORRUNTRIALINPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorRunTrialInput)
  })
_sym_db.RegisterMessage(ActorRunTrialInput)

ActorRunTrialOutput = _reflection.GeneratedProtocolMessageType('ActorRunTrialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORRUNTRIALOUTPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorRunTrialOutput)
  })
_sym_db.RegisterMessage(ActorRunTrialOutput)

ActorInitialInput = _reflection.GeneratedProtocolMessageType('ActorInitialInput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORINITIALINPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorInitialInput)
  })
_sym_db.RegisterMessage(ActorInitialInput)

ActorInitialOutput = _reflection.GeneratedProtocolMessageType('ActorInitialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORINITIALOUTPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorInitialOutput)
  })
_sym_db.RegisterMessage(ActorInitialOutput)

TrialConfig = _reflection.GeneratedProtocolMessageType('TrialConfig', (_message.Message,), {
  'DESCRIPTOR' : _TRIALCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialConfig)
  })
_sym_db.RegisterMessage(TrialConfig)

ActorConfig = _reflection.GeneratedProtocolMessageType('ActorConfig', (_message.Message,), {
  'DESCRIPTOR' : _ACTORCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorConfig)
  })
_sym_db.RegisterMessage(ActorConfig)

EnvironmentConfig = _reflection.GeneratedProtocolMessageType('EnvironmentConfig', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvironmentConfig)
  })
_sym_db.RegisterMessage(EnvironmentConfig)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _TRIALSTATE._serialized_start=2356
  _TRIALSTATE._serialized_end=2453
  _COMMUNICATIONSTATE._serialized_start=2455
  _COMMUNICATIONSTATE._serialized_end=2558
  _VERSIONREQUEST._serialized_start=67
  _VERSIONREQUEST._serialized_end=83
  _VERSIONINFO._serialized_start=85
  _VERSIONINFO._serialized_end=191
  _VERSIONINFO_VERSION._serialized_start=151
  _VERSIONINFO_VERSION._serialized_end=191
  _SERIALIZEDMESSAGE._serialized_start=193
  _SERIALIZEDMESSAGE._serialized_end=229
  _DATALOGPARAMS._serialized_start=231
  _DATALOGPARAMS._serialized_end=288
  _ENVIRONMENTPARAMS._serialized_start=290
  _ENVIRONMENTPARAMS._serialized_end=412
  _ACTORPARAMS._serialized_start=415
  _ACTORPARAMS._serialized_end=687
  _TRIALPARAMS._serialized_start=690
  _TRIALPARAMS._serialized_end=963
  _TRIALACTOR._serialized_start=965
  _TRIALACTOR._serialized_end=1012
  _OBSERVATION._serialized_start=1014
  _OBSERVATION._serialized_end=1080
  _ACTION._serialized_start=1082
  _ACTION._serialized_end=1143
  _REWARDSOURCE._serialized_start=1145
  _REWARDSOURCE._serialized_end=1256
  _REWARD._serialized_start=1258
  _REWARD._serialized_end=1364
  _MESSAGE._serialized_start=1366
  _MESSAGE._serialized_end=1475
  _ACTORRUNTRIALINPUT._serialized_start=1478
  _ACTORRUNTRIALINPUT._serialized_end=1751
  _ACTORRUNTRIALOUTPUT._serialized_start=1754
  _ACTORRUNTRIALOUTPUT._serialized_end=2020
  _ACTORINITIALINPUT._serialized_start=2023
  _ACTORINITIALINPUT._serialized_end=2167
  _ACTORINITIALOUTPUT._serialized_start=2169
  _ACTORINITIALOUTPUT._serialized_end=2252
  _TRIALCONFIG._serialized_start=2254
  _TRIALCONFIG._serialized_end=2284
  _ACTORCONFIG._serialized_start=2286
  _ACTORCONFIG._serialized_end=2316
  _ENVIRONMENTCONFIG._serialized_start=2318
  _ENVIRONMENTCONFIG._serialized_end=2354
# @@protoc_insertion_point(module_scope)
