# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/environment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x63ogment/api/environment.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"^\n\x0eObservationSet\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x14\n\x0cobservations\x18\x03 \x03(\x0c\x12\x12\n\nactors_map\x18\x04 \x03(\x05\"\\\n\tActionSet\x12\x0f\n\x07tick_id\x18\x01 \x01(\x04\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x0f\n\x07\x61\x63tions\x18\x03 \x03(\x0c\x12\x1a\n\x12unavailable_actors\x18\x04 \x03(\r\"\xe4\x01\n\x10\x45nvRunTrialInput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x31\n\ninit_input\x18\x02 \x01(\x0b\x32\x1b.cogmentAPI.EnvInitialInputH\x00\x12+\n\naction_set\x18\x03 \x01(\x0b\x32\x15.cogmentAPI.ActionSetH\x00\x12&\n\x07message\x18\x04 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x05 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\x97\x02\n\x11\x45nvRunTrialOutput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x33\n\x0binit_output\x18\x02 \x01(\x0b\x32\x1c.cogmentAPI.EnvInitialOutputH\x00\x12\x35\n\x0fobservation_set\x18\x03 \x01(\x0b\x32\x1a.cogmentAPI.ObservationSetH\x00\x12$\n\x06reward\x18\x04 \x01(\x0b\x32\x12.cogmentAPI.RewardH\x00\x12&\n\x07message\x18\x05 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x06 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\xa3\x01\n\x0f\x45nvInitialInput\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\timpl_name\x18\x02 \x01(\t\x12\x0f\n\x07tick_id\x18\x03 \x01(\x04\x12/\n\x0f\x61\x63tors_in_trial\x18\x04 \x03(\x0b\x32\x16.cogmentAPI.TrialActor\x12-\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessage\"\x12\n\x10\x45nvInitialOutput2\xa0\x01\n\rEnvironmentSP\x12M\n\x08RunTrial\x12\x1c.cogmentAPI.EnvRunTrialInput\x1a\x1d.cogmentAPI.EnvRunTrialOutput\"\x00(\x01\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')



_OBSERVATIONSET = DESCRIPTOR.message_types_by_name['ObservationSet']
_ACTIONSET = DESCRIPTOR.message_types_by_name['ActionSet']
_ENVRUNTRIALINPUT = DESCRIPTOR.message_types_by_name['EnvRunTrialInput']
_ENVRUNTRIALOUTPUT = DESCRIPTOR.message_types_by_name['EnvRunTrialOutput']
_ENVINITIALINPUT = DESCRIPTOR.message_types_by_name['EnvInitialInput']
_ENVINITIALOUTPUT = DESCRIPTOR.message_types_by_name['EnvInitialOutput']
ObservationSet = _reflection.GeneratedProtocolMessageType('ObservationSet', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATIONSET,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ObservationSet)
  })
_sym_db.RegisterMessage(ObservationSet)

ActionSet = _reflection.GeneratedProtocolMessageType('ActionSet', (_message.Message,), {
  'DESCRIPTOR' : _ACTIONSET,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActionSet)
  })
_sym_db.RegisterMessage(ActionSet)

EnvRunTrialInput = _reflection.GeneratedProtocolMessageType('EnvRunTrialInput', (_message.Message,), {
  'DESCRIPTOR' : _ENVRUNTRIALINPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvRunTrialInput)
  })
_sym_db.RegisterMessage(EnvRunTrialInput)

EnvRunTrialOutput = _reflection.GeneratedProtocolMessageType('EnvRunTrialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ENVRUNTRIALOUTPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvRunTrialOutput)
  })
_sym_db.RegisterMessage(EnvRunTrialOutput)

EnvInitialInput = _reflection.GeneratedProtocolMessageType('EnvInitialInput', (_message.Message,), {
  'DESCRIPTOR' : _ENVINITIALINPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvInitialInput)
  })
_sym_db.RegisterMessage(EnvInitialInput)

EnvInitialOutput = _reflection.GeneratedProtocolMessageType('EnvInitialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ENVINITIALOUTPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvInitialOutput)
  })
_sym_db.RegisterMessage(EnvInitialOutput)

_ENVIRONMENTSP = DESCRIPTOR.services_by_name['EnvironmentSP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _OBSERVATIONSET._serialized_start=71
  _OBSERVATIONSET._serialized_end=165
  _ACTIONSET._serialized_start=167
  _ACTIONSET._serialized_end=259
  _ENVRUNTRIALINPUT._serialized_start=262
  _ENVRUNTRIALINPUT._serialized_end=490
  _ENVRUNTRIALOUTPUT._serialized_start=493
  _ENVRUNTRIALOUTPUT._serialized_end=772
  _ENVINITIALINPUT._serialized_start=775
  _ENVINITIALINPUT._serialized_end=938
  _ENVINITIALOUTPUT._serialized_start=940
  _ENVINITIALOUTPUT._serialized_end=958
  _ENVIRONMENTSP._serialized_start=961
  _ENVIRONMENTSP._serialized_end=1121
# @@protoc_insertion_point(module_scope)
