# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/orchestrator.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2
from cogment.api import environment_pb2 as cogment_dot_api_dot_environment__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x63ogment/api/orchestrator.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\x1a\x1d\x63ogment/api/environment.proto\"\xaa\x01\n\x11TrialStartRequest\x12/\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x1d.cogmentAPI.SerializedMessageH\x00\x12)\n\x06params\x18\x04 \x01(\x0b\x32\x17.cogmentAPI.TrialParamsH\x00\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x1a\n\x12trial_id_requested\x18\x03 \x01(\tB\x0c\n\nstart_data\"#\n\x0fTrialStartReply\x12\x10\n\x08trial_id\x18\x01 \x01(\t\"1\n\x15TerminateTrialRequest\x12\x18\n\x10hard_termination\x18\x01 \x01(\x08\"\x15\n\x13TerminateTrialReply\"J\n\x10TrialInfoRequest\x12\x1e\n\x16get_latest_observation\x18\x01 \x01(\x08\x12\x16\n\x0eget_actor_list\x18\x02 \x01(\x08\"\xe8\x01\n\tTrialInfo\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12%\n\x05state\x18\x02 \x01(\x0e\x32\x16.cogmentAPI.TrialState\x12\x10\n\x08\x65nv_name\x18\x03 \x01(\t\x12\x0f\n\x07tick_id\x18\x04 \x01(\x04\x12\x16\n\x0etrial_duration\x18\x05 \x01(\x06\x12\x36\n\x12latest_observation\x18\x06 \x01(\x0b\x32\x1a.cogmentAPI.ObservationSet\x12/\n\x0f\x61\x63tors_in_trial\x18\x07 \x03(\x0b\x32\x16.cogmentAPI.TrialActor\"6\n\x0eTrialInfoReply\x12$\n\x05trial\x18\x01 \x03(\x0b\x32\x15.cogmentAPI.TrialInfo\":\n\x10TrialListRequest\x12&\n\x06\x66ilter\x18\x01 \x03(\x0e\x32\x16.cogmentAPI.TrialState\"I\n\x0eTrialListEntry\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12%\n\x05state\x18\x02 \x01(\x0e\x32\x16.cogmentAPI.TrialState2\x91\x03\n\x10TrialLifecycleSP\x12J\n\nStartTrial\x12\x1d.cogmentAPI.TrialStartRequest\x1a\x1b.cogmentAPI.TrialStartReply\"\x00\x12V\n\x0eTerminateTrial\x12!.cogmentAPI.TerminateTrialRequest\x1a\x1f.cogmentAPI.TerminateTrialReply\"\x00\x12J\n\x0cGetTrialInfo\x12\x1c.cogmentAPI.TrialInfoRequest\x1a\x1a.cogmentAPI.TrialInfoReply\"\x00\x12K\n\x0bWatchTrials\x12\x1c.cogmentAPI.TrialListRequest\x1a\x1a.cogmentAPI.TrialListEntry\"\x00\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x32\xa4\x01\n\rClientActorSP\x12Q\n\x08RunTrial\x12\x1f.cogmentAPI.ActorRunTrialOutput\x1a\x1e.cogmentAPI.ActorRunTrialInput\"\x00(\x01\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')



_TRIALSTARTREQUEST = DESCRIPTOR.message_types_by_name['TrialStartRequest']
_TRIALSTARTREPLY = DESCRIPTOR.message_types_by_name['TrialStartReply']
_TERMINATETRIALREQUEST = DESCRIPTOR.message_types_by_name['TerminateTrialRequest']
_TERMINATETRIALREPLY = DESCRIPTOR.message_types_by_name['TerminateTrialReply']
_TRIALINFOREQUEST = DESCRIPTOR.message_types_by_name['TrialInfoRequest']
_TRIALINFO = DESCRIPTOR.message_types_by_name['TrialInfo']
_TRIALINFOREPLY = DESCRIPTOR.message_types_by_name['TrialInfoReply']
_TRIALLISTREQUEST = DESCRIPTOR.message_types_by_name['TrialListRequest']
_TRIALLISTENTRY = DESCRIPTOR.message_types_by_name['TrialListEntry']
TrialStartRequest = _reflection.GeneratedProtocolMessageType('TrialStartRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALSTARTREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialStartRequest)
  })
_sym_db.RegisterMessage(TrialStartRequest)

TrialStartReply = _reflection.GeneratedProtocolMessageType('TrialStartReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALSTARTREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialStartReply)
  })
_sym_db.RegisterMessage(TrialStartReply)

TerminateTrialRequest = _reflection.GeneratedProtocolMessageType('TerminateTrialRequest', (_message.Message,), {
  'DESCRIPTOR' : _TERMINATETRIALREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TerminateTrialRequest)
  })
_sym_db.RegisterMessage(TerminateTrialRequest)

TerminateTrialReply = _reflection.GeneratedProtocolMessageType('TerminateTrialReply', (_message.Message,), {
  'DESCRIPTOR' : _TERMINATETRIALREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TerminateTrialReply)
  })
_sym_db.RegisterMessage(TerminateTrialReply)

TrialInfoRequest = _reflection.GeneratedProtocolMessageType('TrialInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALINFOREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialInfoRequest)
  })
_sym_db.RegisterMessage(TrialInfoRequest)

TrialInfo = _reflection.GeneratedProtocolMessageType('TrialInfo', (_message.Message,), {
  'DESCRIPTOR' : _TRIALINFO,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialInfo)
  })
_sym_db.RegisterMessage(TrialInfo)

TrialInfoReply = _reflection.GeneratedProtocolMessageType('TrialInfoReply', (_message.Message,), {
  'DESCRIPTOR' : _TRIALINFOREPLY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialInfoReply)
  })
_sym_db.RegisterMessage(TrialInfoReply)

TrialListRequest = _reflection.GeneratedProtocolMessageType('TrialListRequest', (_message.Message,), {
  'DESCRIPTOR' : _TRIALLISTREQUEST,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialListRequest)
  })
_sym_db.RegisterMessage(TrialListRequest)

TrialListEntry = _reflection.GeneratedProtocolMessageType('TrialListEntry', (_message.Message,), {
  'DESCRIPTOR' : _TRIALLISTENTRY,
  '__module__' : 'cogment.api.orchestrator_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialListEntry)
  })
_sym_db.RegisterMessage(TrialListEntry)

_TRIALLIFECYCLESP = DESCRIPTOR.services_by_name['TrialLifecycleSP']
_CLIENTACTORSP = DESCRIPTOR.services_by_name['ClientActorSP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _TRIALSTARTREQUEST._serialized_start=104
  _TRIALSTARTREQUEST._serialized_end=274
  _TRIALSTARTREPLY._serialized_start=276
  _TRIALSTARTREPLY._serialized_end=311
  _TERMINATETRIALREQUEST._serialized_start=313
  _TERMINATETRIALREQUEST._serialized_end=362
  _TERMINATETRIALREPLY._serialized_start=364
  _TERMINATETRIALREPLY._serialized_end=385
  _TRIALINFOREQUEST._serialized_start=387
  _TRIALINFOREQUEST._serialized_end=461
  _TRIALINFO._serialized_start=464
  _TRIALINFO._serialized_end=696
  _TRIALINFOREPLY._serialized_start=698
  _TRIALINFOREPLY._serialized_end=752
  _TRIALLISTREQUEST._serialized_start=754
  _TRIALLISTREQUEST._serialized_end=812
  _TRIALLISTENTRY._serialized_start=814
  _TRIALLISTENTRY._serialized_end=887
  _TRIALLIFECYCLESP._serialized_start=890
  _TRIALLIFECYCLESP._serialized_end=1291
  _CLIENTACTORSP._serialized_start=1294
  _CLIENTACTORSP._serialized_end=1458
# @@protoc_insertion_point(module_scope)
