# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/trial_datastore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!cogment/api/trial_datastore.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"g\n\x15RetrieveTrialsRequest\x12\x11\n\ttrial_ids\x18\x01 \x03(\t\x12\x0f\n\x07timeout\x18\x02 \x01(\r\x12\x14\n\x0ctrials_count\x18\x03 \x01(\r\x12\x14\n\x0ctrial_handle\x18\x04 \x01(\t\"b\n\x13RetrieveTrialsReply\x12\x30\n\x0btrial_infos\x18\x01 \x03(\x0b\x32\x1b.cogmentAPI.StoredTrialInfo\x12\x19\n\x11next_trial_handle\x18\x02 \x01(\t\"\xba\x01\n\x16RetrieveSamplesRequest\x12\x11\n\ttrial_ids\x18\x01 \x03(\t\x12\x13\n\x0b\x61\x63tor_names\x18\x02 \x03(\t\x12\x15\n\ractor_classes\x18\x03 \x03(\t\x12\x1d\n\x15\x61\x63tor_implementations\x18\x04 \x03(\t\x12\x42\n\x16selected_sample_fields\x18\x05 \x03(\x0e\x32\".cogmentAPI.StoredTrialSampleField\"J\n\x13RetrieveSampleReply\x12\x33\n\x0ctrial_sample\x18\x01 \x01(\x0b\x32\x1d.cogmentAPI.StoredTrialSample\"(\n\x13\x44\x65leteTrialsRequest\x12\x11\n\ttrial_ids\x18\x01 \x03(\t\"\x13\n\x11\x44\x65leteTrialsReply\"Q\n\x0f\x41\x64\x64TrialRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12-\n\x0ctrial_params\x18\x02 \x01(\x0b\x32\x17.cogmentAPI.TrialParams\"\x0f\n\rAddTrialReply\"G\n\x10\x41\x64\x64SampleRequest\x12\x33\n\x0ctrial_sample\x18\x01 \x01(\x0b\x32\x1d.cogmentAPI.StoredTrialSample\"\x11\n\x0f\x41\x64\x64SamplesReply\"\xa0\x01\n\x0fStoredTrialInfo\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12*\n\nlast_state\x18\x02 \x01(\x0e\x32\x16.cogmentAPI.TrialState\x12\x0f\n\x07user_id\x18\x03 \x01(\t\x12\x15\n\rsamples_count\x18\x04 \x01(\r\x12\'\n\x06params\x18\x05 \x01(\x0b\x32\x17.cogmentAPI.TrialParams\"\xce\x01\n\x11StoredTrialSample\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x10\n\x08trial_id\x18\x02 \x01(\t\x12\x0f\n\x07tick_id\x18\x03 \x01(\x04\x12\x11\n\ttimestamp\x18\x04 \x01(\x06\x12%\n\x05state\x18\x05 \x01(\x0e\x32\x16.cogmentAPI.TrialState\x12\x39\n\ractor_samples\x18\x06 \x03(\x0b\x32\".cogmentAPI.StoredTrialActorSample\x12\x10\n\x08payloads\x18\x07 \x03(\x0c\"\x9d\x03\n\x16StoredTrialActorSample\x12\r\n\x05\x61\x63tor\x18\x01 \x01(\r\x12\x18\n\x0bobservation\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x13\n\x06\x61\x63tion\x18\x03 \x01(\rH\x01\x88\x01\x01\x12\x13\n\x06reward\x18\x04 \x01(\x02H\x02\x88\x01\x01\x12\x42\n\x10received_rewards\x18\x06 \x03(\x0b\x32(.cogmentAPI.StoredTrialActorSampleReward\x12>\n\x0csent_rewards\x18\x07 \x03(\x0b\x32(.cogmentAPI.StoredTrialActorSampleReward\x12\x44\n\x11received_messages\x18\x08 \x03(\x0b\x32).cogmentAPI.StoredTrialActorSampleMessage\x12@\n\rsent_messages\x18\t \x03(\x0b\x32).cogmentAPI.StoredTrialActorSampleMessageB\x0e\n\x0c_observationB\t\n\x07_actionB\t\n\x07_reward\"\x8a\x01\n\x1cStoredTrialActorSampleReward\x12\x0e\n\x06sender\x18\x01 \x01(\x05\x12\x10\n\x08receiver\x18\x02 \x01(\x05\x12\x0e\n\x06reward\x18\x04 \x01(\x02\x12\x12\n\nconfidence\x18\x05 \x01(\x02\x12\x16\n\tuser_data\x18\x06 \x01(\rH\x00\x88\x01\x01\x42\x0c\n\n_user_data\"R\n\x1dStoredTrialActorSampleMessage\x12\x0e\n\x06sender\x18\x01 \x01(\x05\x12\x10\n\x08receiver\x18\x02 \x01(\x05\x12\x0f\n\x07payload\x18\x03 \x01(\r\"\xf9\x01\n\x16TrialSamplesFileHeader\x12-\n\x0cversion_info\x18\x01 \x01(\x0b\x32\x17.cogmentAPI.VersionInfo\x12\x18\n\x10\x65xport_timestamp\x18\x02 \x01(\x06\x12I\n\x0ctrial_params\x18\x03 \x03(\x0b\x32\x33.cogmentAPI.TrialSamplesFileHeader.TrialParamsEntry\x1aK\n\x10TrialParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.cogmentAPI.TrialParams:\x02\x38\x01*\xf0\x02\n\x16StoredTrialSampleField\x12%\n!STORED_TRIAL_SAMPLE_FIELD_UNKNOWN\x10\x00\x12)\n%STORED_TRIAL_SAMPLE_FIELD_OBSERVATION\x10\x01\x12$\n STORED_TRIAL_SAMPLE_FIELD_ACTION\x10\x02\x12$\n STORED_TRIAL_SAMPLE_FIELD_REWARD\x10\x03\x12.\n*STORED_TRIAL_SAMPLE_FIELD_RECEIVED_REWARDS\x10\x04\x12*\n&STORED_TRIAL_SAMPLE_FIELD_SENT_REWARDS\x10\x05\x12/\n+STORED_TRIAL_SAMPLE_FIELD_RECEIVED_MESSAGES\x10\x06\x12+\n\'STORED_TRIAL_SAMPLE_FIELD_SENT_MESSAGES\x10\x07\x32\xec\x03\n\x10TrialDatastoreSP\x12V\n\x0eRetrieveTrials\x12!.cogmentAPI.RetrieveTrialsRequest\x1a\x1f.cogmentAPI.RetrieveTrialsReply\"\x00\x12Z\n\x0fRetrieveSamples\x12\".cogmentAPI.RetrieveSamplesRequest\x1a\x1f.cogmentAPI.RetrieveSampleReply\"\x00\x30\x01\x12\x44\n\x08\x41\x64\x64Trial\x12\x1b.cogmentAPI.AddTrialRequest\x1a\x19.cogmentAPI.AddTrialReply\"\x00\x12J\n\tAddSample\x12\x1c.cogmentAPI.AddSampleRequest\x1a\x1b.cogmentAPI.AddSamplesReply\"\x00(\x01\x12P\n\x0c\x44\x65leteTrials\x12\x1f.cogmentAPI.DeleteTrialsRequest\x1a\x1d.cogmentAPI.DeleteTrialsReply\"\x00\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')

_STOREDTRIALSAMPLEFIELD = DESCRIPTOR.enum_types_by_name['StoredTrialSampleField']
StoredTrialSampleField = enum_type_wrapper.EnumTypeWrapper(_STOREDTRIALSAMPLEFIELD)
STORED_TRIAL_SAMPLE_FIELD_UNKNOWN = 0
STORED_TRIAL_SAMPLE_FIELD_OBSERVATION = 1
STORED_TRIAL_SAMPLE_FIELD_ACTION = 2
STORED_TRIAL_SAMPLE_FIELD_REWARD = 3
STORED_TRIAL_SAMPLE_FIELD_RECEIVED_REWARDS = 4
STORED_TRIAL_SAMPLE_FIELD_SENT_REWARDS = 5
STORED_TRIAL_SAMPLE_FIELD_RECEIVED_MESSAGES = 6
STORED_TRIAL_SAMPLE_FIELD_SENT_MESSAGES = 7


_RETRIEVETRIALSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveTrialsRequest']
_RETRIEVETRIALSREPLY = DESCRIPTOR.message_types_by_name['RetrieveTrialsReply']
_RETRIEVESAMPLESREQUEST = DESCRIPTOR.message_types_by_name['RetrieveSamplesRequest']
_RETRIEVESAMPLEREPLY = DESCRIPTOR.message_types_by_name['RetrieveSampleReply']
_DELETETRIALSREQUEST = DESCRIPTOR.message_types_by_name['DeleteTrialsRequest']
_DELETETRIALSREPLY = DESCRIPTOR.message_types_by_name['DeleteTrialsReply']
_ADDTRIALREQUEST = DESCRIPTOR.message_types_by_name['AddTrialRequest']
_ADDTRIALREPLY = DESCRIPTOR.message_types_by_name['AddTrialReply']
_ADDSAMPLEREQUEST = DESCRIPTOR.message_types_by_name['AddSampleRequest']
_ADDSAMPLESREPLY = DESCRIPTOR.message_types_by_name['AddSamplesReply']
_STOREDTRIALINFO = DESCRIPTOR.message_types_by_name['StoredTrialInfo']
_STOREDTRIALSAMPLE = DESCRIPTOR.message_types_by_name['StoredTrialSample']
_STOREDTRIALACTORSAMPLE = DESCRIPTOR.message_types_by_name['StoredTrialActorSample']
_STOREDTRIALACTORSAMPLEREWARD = DESCRIPTOR.message_types_by_name['StoredTrialActorSampleReward']
_STOREDTRIALACTORSAMPLEMESSAGE = DESCRIPTOR.message_types_by_name['StoredTrialActorSampleMessage']
_TRIALSAMPLESFILEHEADER = DESCRIPTOR.message_types_by_name['TrialSamplesFileHeader']
_TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY = _TRIALSAMPLESFILEHEADER.nested_types_by_name['TrialParamsEntry']
RetrieveTrialsRequest = _reflection.GeneratedProtocolMessageType('RetrieveTrialsRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVETRIALSREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveTrialsRequest)
  })
_sym_db.RegisterMessage(RetrieveTrialsRequest)

RetrieveTrialsReply = _reflection.GeneratedProtocolMessageType('RetrieveTrialsReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVETRIALSREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveTrialsReply)
  })
_sym_db.RegisterMessage(RetrieveTrialsReply)

RetrieveSamplesRequest = _reflection.GeneratedProtocolMessageType('RetrieveSamplesRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVESAMPLESREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveSamplesRequest)
  })
_sym_db.RegisterMessage(RetrieveSamplesRequest)

RetrieveSampleReply = _reflection.GeneratedProtocolMessageType('RetrieveSampleReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVESAMPLEREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveSampleReply)
  })
_sym_db.RegisterMessage(RetrieveSampleReply)

DeleteTrialsRequest = _reflection.GeneratedProtocolMessageType('DeleteTrialsRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETETRIALSREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteTrialsRequest)
  })
_sym_db.RegisterMessage(DeleteTrialsRequest)

DeleteTrialsReply = _reflection.GeneratedProtocolMessageType('DeleteTrialsReply', (_message.Message,), {
  'DESCRIPTOR' : _DELETETRIALSREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteTrialsReply)
  })
_sym_db.RegisterMessage(DeleteTrialsReply)

AddTrialRequest = _reflection.GeneratedProtocolMessageType('AddTrialRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDTRIALREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddTrialRequest)
  })
_sym_db.RegisterMessage(AddTrialRequest)

AddTrialReply = _reflection.GeneratedProtocolMessageType('AddTrialReply', (_message.Message,), {
  'DESCRIPTOR' : _ADDTRIALREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddTrialReply)
  })
_sym_db.RegisterMessage(AddTrialReply)

AddSampleRequest = _reflection.GeneratedProtocolMessageType('AddSampleRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDSAMPLEREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddSampleRequest)
  })
_sym_db.RegisterMessage(AddSampleRequest)

AddSamplesReply = _reflection.GeneratedProtocolMessageType('AddSamplesReply', (_message.Message,), {
  'DESCRIPTOR' : _ADDSAMPLESREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddSamplesReply)
  })
_sym_db.RegisterMessage(AddSamplesReply)

StoredTrialInfo = _reflection.GeneratedProtocolMessageType('StoredTrialInfo', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALINFO,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialInfo)
  })
_sym_db.RegisterMessage(StoredTrialInfo)

StoredTrialSample = _reflection.GeneratedProtocolMessageType('StoredTrialSample', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALSAMPLE,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialSample)
  })
_sym_db.RegisterMessage(StoredTrialSample)

StoredTrialActorSample = _reflection.GeneratedProtocolMessageType('StoredTrialActorSample', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALACTORSAMPLE,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialActorSample)
  })
_sym_db.RegisterMessage(StoredTrialActorSample)

StoredTrialActorSampleReward = _reflection.GeneratedProtocolMessageType('StoredTrialActorSampleReward', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALACTORSAMPLEREWARD,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialActorSampleReward)
  })
_sym_db.RegisterMessage(StoredTrialActorSampleReward)

StoredTrialActorSampleMessage = _reflection.GeneratedProtocolMessageType('StoredTrialActorSampleMessage', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALACTORSAMPLEMESSAGE,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialActorSampleMessage)
  })
_sym_db.RegisterMessage(StoredTrialActorSampleMessage)

TrialSamplesFileHeader = _reflection.GeneratedProtocolMessageType('TrialSamplesFileHeader', (_message.Message,), {

  'TrialParamsEntry' : _reflection.GeneratedProtocolMessageType('TrialParamsEntry', (_message.Message,), {
    'DESCRIPTOR' : _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY,
    '__module__' : 'cogment.api.trial_datastore_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.TrialSamplesFileHeader.TrialParamsEntry)
    })
  ,
  'DESCRIPTOR' : _TRIALSAMPLESFILEHEADER,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialSamplesFileHeader)
  })
_sym_db.RegisterMessage(TrialSamplesFileHeader)
_sym_db.RegisterMessage(TrialSamplesFileHeader.TrialParamsEntry)

_TRIALDATASTORESP = DESCRIPTOR.services_by_name['TrialDatastoreSP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._options = None
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._serialized_options = b'8\001'
  _STOREDTRIALSAMPLEFIELD._serialized_start=2066
  _STOREDTRIALSAMPLEFIELD._serialized_end=2434
  _RETRIEVETRIALSREQUEST._serialized_start=75
  _RETRIEVETRIALSREQUEST._serialized_end=178
  _RETRIEVETRIALSREPLY._serialized_start=180
  _RETRIEVETRIALSREPLY._serialized_end=278
  _RETRIEVESAMPLESREQUEST._serialized_start=281
  _RETRIEVESAMPLESREQUEST._serialized_end=467
  _RETRIEVESAMPLEREPLY._serialized_start=469
  _RETRIEVESAMPLEREPLY._serialized_end=543
  _DELETETRIALSREQUEST._serialized_start=545
  _DELETETRIALSREQUEST._serialized_end=585
  _DELETETRIALSREPLY._serialized_start=587
  _DELETETRIALSREPLY._serialized_end=606
  _ADDTRIALREQUEST._serialized_start=608
  _ADDTRIALREQUEST._serialized_end=689
  _ADDTRIALREPLY._serialized_start=691
  _ADDTRIALREPLY._serialized_end=706
  _ADDSAMPLEREQUEST._serialized_start=708
  _ADDSAMPLEREQUEST._serialized_end=779
  _ADDSAMPLESREPLY._serialized_start=781
  _ADDSAMPLESREPLY._serialized_end=798
  _STOREDTRIALINFO._serialized_start=801
  _STOREDTRIALINFO._serialized_end=961
  _STOREDTRIALSAMPLE._serialized_start=964
  _STOREDTRIALSAMPLE._serialized_end=1170
  _STOREDTRIALACTORSAMPLE._serialized_start=1173
  _STOREDTRIALACTORSAMPLE._serialized_end=1586
  _STOREDTRIALACTORSAMPLEREWARD._serialized_start=1589
  _STOREDTRIALACTORSAMPLEREWARD._serialized_end=1727
  _STOREDTRIALACTORSAMPLEMESSAGE._serialized_start=1729
  _STOREDTRIALACTORSAMPLEMESSAGE._serialized_end=1811
  _TRIALSAMPLESFILEHEADER._serialized_start=1814
  _TRIALSAMPLESFILEHEADER._serialized_end=2063
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._serialized_start=1988
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._serialized_end=2063
  _TRIALDATASTORESP._serialized_start=2437
  _TRIALDATASTORESP._serialized_end=2929
# @@protoc_insertion_point(module_scope)
