# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/model_registry.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n cogment/api/model_registry.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"\x86\x01\n\tModelInfo\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x36\n\tuser_data\x18\x02 \x03(\x0b\x32#.cogmentAPI.ModelInfo.UserDataEntry\x1a/\n\rUserDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x80\x02\n\x10ModelVersionInfo\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x16\n\x0eversion_number\x18\x02 \x01(\r\x12\x1a\n\x12\x63reation_timestamp\x18\x03 \x01(\x06\x12\x10\n\x08\x61rchived\x18\x04 \x01(\x08\x12\x11\n\tdata_hash\x18\x05 \x01(\t\x12\x11\n\tdata_size\x18\x06 \x01(\x06\x12=\n\tuser_data\x18\x07 \x03(\x0b\x32*.cogmentAPI.ModelVersionInfo.UserDataEntry\x1a/\n\rUserDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"G\n\x1a\x43reateOrUpdateModelRequest\x12)\n\nmodel_info\x18\x01 \x01(\x0b\x32\x15.cogmentAPI.ModelInfo\"\x1a\n\x18\x43reateOrUpdateModelReply\"V\n\x15RetrieveModelsRequest\x12\x11\n\tmodel_ids\x18\x01 \x03(\t\x12\x14\n\x0cmodels_count\x18\x03 \x01(\r\x12\x14\n\x0cmodel_handle\x18\x04 \x01(\t\"\\\n\x13RetrieveModelsReply\x12*\n\x0bmodel_infos\x18\x01 \x03(\x0b\x32\x15.cogmentAPI.ModelInfo\x12\x19\n\x11next_model_handle\x18\x02 \x01(\t\"&\n\x12\x44\x65leteModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\"\x12\n\x10\x44\x65leteModelReply\"\xf8\x01\n\x19\x43reateVersionRequestChunk\x12>\n\x06header\x18\x01 \x01(\x0b\x32,.cogmentAPI.CreateVersionRequestChunk.HeaderH\x00\x12:\n\x04\x62ody\x18\x02 \x01(\x0b\x32*.cogmentAPI.CreateVersionRequestChunk.BodyH\x00\x1a<\n\x06Header\x12\x32\n\x0cversion_info\x18\x01 \x01(\x0b\x32\x1c.cogmentAPI.ModelVersionInfo\x1a\x1a\n\x04\x42ody\x12\x12\n\ndata_chunk\x18\x01 \x01(\x0c\x42\x05\n\x03msg\"H\n\x12\x43reateVersionReply\x12\x32\n\x0cversion_info\x18\x01 \x01(\x0b\x32\x1c.cogmentAPI.ModelVersionInfo\"x\n\x1bRetrieveVersionInfosRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x17\n\x0fversion_numbers\x18\x02 \x03(\x05\x12\x16\n\x0eversions_count\x18\x03 \x01(\r\x12\x16\n\x0eversion_handle\x18\x04 \x01(\t\"m\n\x19RetrieveVersionInfosReply\x12\x33\n\rversion_infos\x18\x01 \x03(\x0b\x32\x1c.cogmentAPI.ModelVersionInfo\x12\x1b\n\x13next_version_handle\x18\x02 \x01(\t\"F\n\x1aRetrieveVersionDataRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x16\n\x0eversion_number\x18\x02 \x01(\x05\"3\n\x1dRetrieveVersionDataReplyChunk\x12\x12\n\ndata_chunk\x18\x01 \x01(\x0c\x32\x95\x05\n\x0fModelRegistrySP\x12\x65\n\x13\x43reateOrUpdateModel\x12&.cogmentAPI.CreateOrUpdateModelRequest\x1a$.cogmentAPI.CreateOrUpdateModelReply\"\x00\x12M\n\x0b\x44\x65leteModel\x12\x1e.cogmentAPI.DeleteModelRequest\x1a\x1c.cogmentAPI.DeleteModelReply\"\x00\x12V\n\x0eRetrieveModels\x12!.cogmentAPI.RetrieveModelsRequest\x1a\x1f.cogmentAPI.RetrieveModelsReply\"\x00\x12Z\n\rCreateVersion\x12%.cogmentAPI.CreateVersionRequestChunk\x1a\x1e.cogmentAPI.CreateVersionReply\"\x00(\x01\x12h\n\x14RetrieveVersionInfos\x12\'.cogmentAPI.RetrieveVersionInfosRequest\x1a%.cogmentAPI.RetrieveVersionInfosReply\"\x00\x12l\n\x13RetrieveVersionData\x12&.cogmentAPI.RetrieveVersionDataRequest\x1a).cogmentAPI.RetrieveVersionDataReplyChunk\"\x00\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')



_MODELINFO = DESCRIPTOR.message_types_by_name['ModelInfo']
_MODELINFO_USERDATAENTRY = _MODELINFO.nested_types_by_name['UserDataEntry']
_MODELVERSIONINFO = DESCRIPTOR.message_types_by_name['ModelVersionInfo']
_MODELVERSIONINFO_USERDATAENTRY = _MODELVERSIONINFO.nested_types_by_name['UserDataEntry']
_CREATEORUPDATEMODELREQUEST = DESCRIPTOR.message_types_by_name['CreateOrUpdateModelRequest']
_CREATEORUPDATEMODELREPLY = DESCRIPTOR.message_types_by_name['CreateOrUpdateModelReply']
_RETRIEVEMODELSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveModelsRequest']
_RETRIEVEMODELSREPLY = DESCRIPTOR.message_types_by_name['RetrieveModelsReply']
_DELETEMODELREQUEST = DESCRIPTOR.message_types_by_name['DeleteModelRequest']
_DELETEMODELREPLY = DESCRIPTOR.message_types_by_name['DeleteModelReply']
_CREATEVERSIONREQUESTCHUNK = DESCRIPTOR.message_types_by_name['CreateVersionRequestChunk']
_CREATEVERSIONREQUESTCHUNK_HEADER = _CREATEVERSIONREQUESTCHUNK.nested_types_by_name['Header']
_CREATEVERSIONREQUESTCHUNK_BODY = _CREATEVERSIONREQUESTCHUNK.nested_types_by_name['Body']
_CREATEVERSIONREPLY = DESCRIPTOR.message_types_by_name['CreateVersionReply']
_RETRIEVEVERSIONINFOSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveVersionInfosRequest']
_RETRIEVEVERSIONINFOSREPLY = DESCRIPTOR.message_types_by_name['RetrieveVersionInfosReply']
_RETRIEVEVERSIONDATAREQUEST = DESCRIPTOR.message_types_by_name['RetrieveVersionDataRequest']
_RETRIEVEVERSIONDATAREPLYCHUNK = DESCRIPTOR.message_types_by_name['RetrieveVersionDataReplyChunk']
ModelInfo = _reflection.GeneratedProtocolMessageType('ModelInfo', (_message.Message,), {

  'UserDataEntry' : _reflection.GeneratedProtocolMessageType('UserDataEntry', (_message.Message,), {
    'DESCRIPTOR' : _MODELINFO_USERDATAENTRY,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.ModelInfo.UserDataEntry)
    })
  ,
  'DESCRIPTOR' : _MODELINFO,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ModelInfo)
  })
_sym_db.RegisterMessage(ModelInfo)
_sym_db.RegisterMessage(ModelInfo.UserDataEntry)

ModelVersionInfo = _reflection.GeneratedProtocolMessageType('ModelVersionInfo', (_message.Message,), {

  'UserDataEntry' : _reflection.GeneratedProtocolMessageType('UserDataEntry', (_message.Message,), {
    'DESCRIPTOR' : _MODELVERSIONINFO_USERDATAENTRY,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.ModelVersionInfo.UserDataEntry)
    })
  ,
  'DESCRIPTOR' : _MODELVERSIONINFO,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ModelVersionInfo)
  })
_sym_db.RegisterMessage(ModelVersionInfo)
_sym_db.RegisterMessage(ModelVersionInfo.UserDataEntry)

CreateOrUpdateModelRequest = _reflection.GeneratedProtocolMessageType('CreateOrUpdateModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEORUPDATEMODELREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateOrUpdateModelRequest)
  })
_sym_db.RegisterMessage(CreateOrUpdateModelRequest)

CreateOrUpdateModelReply = _reflection.GeneratedProtocolMessageType('CreateOrUpdateModelReply', (_message.Message,), {
  'DESCRIPTOR' : _CREATEORUPDATEMODELREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateOrUpdateModelReply)
  })
_sym_db.RegisterMessage(CreateOrUpdateModelReply)

RetrieveModelsRequest = _reflection.GeneratedProtocolMessageType('RetrieveModelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEMODELSREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveModelsRequest)
  })
_sym_db.RegisterMessage(RetrieveModelsRequest)

RetrieveModelsReply = _reflection.GeneratedProtocolMessageType('RetrieveModelsReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEMODELSREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveModelsReply)
  })
_sym_db.RegisterMessage(RetrieveModelsReply)

DeleteModelRequest = _reflection.GeneratedProtocolMessageType('DeleteModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteModelRequest)
  })
_sym_db.RegisterMessage(DeleteModelRequest)

DeleteModelReply = _reflection.GeneratedProtocolMessageType('DeleteModelReply', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteModelReply)
  })
_sym_db.RegisterMessage(DeleteModelReply)

CreateVersionRequestChunk = _reflection.GeneratedProtocolMessageType('CreateVersionRequestChunk', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _CREATEVERSIONREQUESTCHUNK_HEADER,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionRequestChunk.Header)
    })
  ,

  'Body' : _reflection.GeneratedProtocolMessageType('Body', (_message.Message,), {
    'DESCRIPTOR' : _CREATEVERSIONREQUESTCHUNK_BODY,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionRequestChunk.Body)
    })
  ,
  'DESCRIPTOR' : _CREATEVERSIONREQUESTCHUNK,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionRequestChunk)
  })
_sym_db.RegisterMessage(CreateVersionRequestChunk)
_sym_db.RegisterMessage(CreateVersionRequestChunk.Header)
_sym_db.RegisterMessage(CreateVersionRequestChunk.Body)

CreateVersionReply = _reflection.GeneratedProtocolMessageType('CreateVersionReply', (_message.Message,), {
  'DESCRIPTOR' : _CREATEVERSIONREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionReply)
  })
_sym_db.RegisterMessage(CreateVersionReply)

RetrieveVersionInfosRequest = _reflection.GeneratedProtocolMessageType('RetrieveVersionInfosRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONINFOSREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionInfosRequest)
  })
_sym_db.RegisterMessage(RetrieveVersionInfosRequest)

RetrieveVersionInfosReply = _reflection.GeneratedProtocolMessageType('RetrieveVersionInfosReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONINFOSREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionInfosReply)
  })
_sym_db.RegisterMessage(RetrieveVersionInfosReply)

RetrieveVersionDataRequest = _reflection.GeneratedProtocolMessageType('RetrieveVersionDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONDATAREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionDataRequest)
  })
_sym_db.RegisterMessage(RetrieveVersionDataRequest)

RetrieveVersionDataReplyChunk = _reflection.GeneratedProtocolMessageType('RetrieveVersionDataReplyChunk', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONDATAREPLYCHUNK,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionDataReplyChunk)
  })
_sym_db.RegisterMessage(RetrieveVersionDataReplyChunk)

_MODELREGISTRYSP = DESCRIPTOR.services_by_name['ModelRegistrySP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _MODELINFO_USERDATAENTRY._options = None
  _MODELINFO_USERDATAENTRY._serialized_options = b'8\001'
  _MODELVERSIONINFO_USERDATAENTRY._options = None
  _MODELVERSIONINFO_USERDATAENTRY._serialized_options = b'8\001'
  _MODELINFO._serialized_start=75
  _MODELINFO._serialized_end=209
  _MODELINFO_USERDATAENTRY._serialized_start=162
  _MODELINFO_USERDATAENTRY._serialized_end=209
  _MODELVERSIONINFO._serialized_start=212
  _MODELVERSIONINFO._serialized_end=468
  _MODELVERSIONINFO_USERDATAENTRY._serialized_start=162
  _MODELVERSIONINFO_USERDATAENTRY._serialized_end=209
  _CREATEORUPDATEMODELREQUEST._serialized_start=470
  _CREATEORUPDATEMODELREQUEST._serialized_end=541
  _CREATEORUPDATEMODELREPLY._serialized_start=543
  _CREATEORUPDATEMODELREPLY._serialized_end=569
  _RETRIEVEMODELSREQUEST._serialized_start=571
  _RETRIEVEMODELSREQUEST._serialized_end=657
  _RETRIEVEMODELSREPLY._serialized_start=659
  _RETRIEVEMODELSREPLY._serialized_end=751
  _DELETEMODELREQUEST._serialized_start=753
  _DELETEMODELREQUEST._serialized_end=791
  _DELETEMODELREPLY._serialized_start=793
  _DELETEMODELREPLY._serialized_end=811
  _CREATEVERSIONREQUESTCHUNK._serialized_start=814
  _CREATEVERSIONREQUESTCHUNK._serialized_end=1062
  _CREATEVERSIONREQUESTCHUNK_HEADER._serialized_start=967
  _CREATEVERSIONREQUESTCHUNK_HEADER._serialized_end=1027
  _CREATEVERSIONREQUESTCHUNK_BODY._serialized_start=1029
  _CREATEVERSIONREQUESTCHUNK_BODY._serialized_end=1055
  _CREATEVERSIONREPLY._serialized_start=1064
  _CREATEVERSIONREPLY._serialized_end=1136
  _RETRIEVEVERSIONINFOSREQUEST._serialized_start=1138
  _RETRIEVEVERSIONINFOSREQUEST._serialized_end=1258
  _RETRIEVEVERSIONINFOSREPLY._serialized_start=1260
  _RETRIEVEVERSIONINFOSREPLY._serialized_end=1369
  _RETRIEVEVERSIONDATAREQUEST._serialized_start=1371
  _RETRIEVEVERSIONDATAREQUEST._serialized_end=1441
  _RETRIEVEVERSIONDATAREPLYCHUNK._serialized_start=1443
  _RETRIEVEVERSIONDATAREPLYCHUNK._serialized_end=1494
  _MODELREGISTRYSP._serialized_start=1497
  _MODELREGISTRYSP._serialized_end=2158
# @@protoc_insertion_point(module_scope)
